@@ -1,6 +1,7 @@
 """Mid-level helper functions for AutoTS."""
 import sys
 import random
+from math import ceil
 import numpy as np
 import pandas as pd
 import datetime
@@ -28,6 +29,7 @@ from autots.models.basics import (
     AverageValueNaive,
     SeasonalNaive,
     ZeroesNaive,
+    Motif,
 )
 from autots.models.statsmodels import (
     GLS,
@@ -400,6 +402,28 @@ def ModelMonster(
             **parameters,
         )
 
+        return model
+    elif model == 'MultivariateMotif':
+        model = Motif(
+            frequency=frequency,
+            prediction_interval=prediction_interval,
+            random_seed=random_seed,
+            verbose=verbose,
+            n_jobs=n_jobs,
+            multivariate=True,
+            **parameters,
+        )
+        return model
+    elif model == 'UnivariateMotif':
+        model = Motif(
+            frequency=frequency,
+            prediction_interval=prediction_interval,
+            random_seed=random_seed,
+            verbose=verbose,
+            n_jobs=n_jobs,
+            multivariate=False,
+            **parameters,
+        )
         return model
     else:
         raise AttributeError(
@@ -1061,6 +1085,7 @@ def TemplateWizard(
                     'Ensemble': ensemble_input,
                     'Exceptions': np.nan,
                     'Runs': 1,
+                    'Generation': current_generation,
                     'ValidationRound': validation_round,
                 },
                 index=[0],
@@ -1137,6 +1162,7 @@ def TemplateWizard(
                         'TotalRuntime': datetime.timedelta(0),
                         'Exceptions': "KeyboardInterrupt by user",
                         'Runs': 1,
+                        'Generation': current_generation,
                         'ValidationRound': validation_round,
                     },
                     index=[0],
@@ -1184,6 +1210,7 @@ def TemplateWizard(
                     'TotalRuntime': datetime.timedelta(0),
                     'Exceptions': repr(e),
                     'Runs': 1,
+                    'Generation': current_generation,
                     'ValidationRound': validation_round,
                 },
                 index=[0],
@@ -1604,6 +1631,11 @@ def generate_score(
     # generate minimizing scores, where smaller = better accuracy
     try:
         model_results = model_results.replace([np.inf, -np.inf], np.nan)
+        # not sure why there are negative SMAPE values, but make sure they get dealt with
+        if model_results['smape'].min() < 0:
+            model_results['smape'] = model_results['smape'].where(
+                model_results['smape'] >= 0, model_results['smape'].max()
+            )
         # handle NaN in scores...
         # model_results = model_results.fillna(value=model_results.max(axis=0))
 
@@ -1698,12 +1730,138 @@ def generate_score_per_series(results_object, metric_weighting, total_validation
             overall_score = overall_score + (contour_score * contour_weighting)
     # remove basic duplicates
     local_results = results_object.model_results.copy()
+    local_results = local_results[local_results['Exceptions'].isna()]
     local_results = local_results.sort_values(by="TotalRuntimeSeconds", ascending=True)
     local_results.drop_duplicates(
         subset=['ValidationRound', 'smape', 'mae', 'spl'], keep="first", inplace=True
     )
     # select only models run through all validations
+    # run_count = temp.groupby(level=0).count().mean(axis=1)
+    # models_to_use = run_count[run_count >= total_validations].index.tolist()
     run_count = local_results[['Model', 'ID']].groupby("ID").count()
     models_to_use = run_count[run_count['Model'] >= total_validations].index.tolist()
     overall_score = overall_score[overall_score.index.isin(models_to_use)]
+    # take the average score across validations
+    overall_score = overall_score.groupby(level=0).mean()
     return overall_score
+
+
+def back_forecast(
+    df,
+    model_name,
+    model_param_dict,
+    model_transform_dict,
+    future_regressor_train=None,
+    n_splits: int = "auto",
+    forecast_length=14,
+    frequency="infer",
+    prediction_interval=0.9,
+    no_negatives=False,
+    constraint=None,
+    holiday_country="US",
+    random_seed=123,
+    n_jobs="auto",
+    verbose=0,
+):
+    """Create forecasts for the historical training data, ie. backcast or back forecast.
+
+    This actually forecasts on historical data, these are not fit model values as are often returned by other packages.
+    As such, this will be slower, but more representative of real world model performance.
+    There may be jumps in data between chunks.
+
+    Args are same as for model_forecast except...
+    n_splits(int): how many pieces to split data into. Pass 2 for fastest, or "auto" for best accuracy
+
+    Returns a standard prediction object (access .forecast, .lower_forecast, .upper_forecast)
+    """
+    max_chunk = int(ceil(df.index.shape[0] / forecast_length))
+    if not str(n_splits).isdigit():
+        n_splits = max_chunk
+    elif n_splits > max_chunk or n_splits < 2:
+        n_splits = max_chunk
+    else:
+        n_splits = int(n_splits)
+
+    chunk_size = df.index.shape[0] / n_splits
+    b_forecast, b_forecast_up, b_forecast_low = (
+        pd.DataFrame(),
+        pd.DataFrame(),
+        pd.DataFrame(),
+    )
+    for n in range(n_splits):
+        int_idx = int(n * chunk_size)
+        int_idx_1 = int((n + 1) * chunk_size)
+        inner_forecast_length = int_idx_1 - int_idx
+        # flip to forecast backwards for the first split
+        if n == 0:
+            df_split = df.iloc[int_idx_1:].copy()
+            df_split = df_split.iloc[::-1]
+            df_split.index = df_split.index[::-1]
+            result_idx = df.iloc[0:int_idx_1].index
+        else:
+            df_split = df.iloc[0:int_idx].copy()
+        # handle appropriate regressors
+        if isinstance(future_regressor_train, pd.DataFrame):
+            if n == 0:
+                split_regr = future_regressor_train.reindex(df_split.index[::-1])
+                split_regr_future = future_regressor_train.reindex(result_idx)
+            else:
+                split_regr = future_regressor_train.reindex(df_split.index)
+                split_regr_future = future_regressor_train.reindex(
+                    df.index[int_idx:int_idx_1]
+                )
+        else:
+            split_regr = []
+            split_regr_future = []
+        try:
+            df_forecast = model_forecast(
+                model_name=model_name,
+                model_param_dict=model_param_dict,
+                model_transform_dict=model_transform_dict,
+                df_train=df_split,
+                forecast_length=inner_forecast_length,
+                frequency=frequency,
+                prediction_interval=prediction_interval,
+                no_negatives=no_negatives,
+                constraint=constraint,
+                future_regressor_train=split_regr,
+                future_regressor_forecast=split_regr_future,
+                holiday_country=holiday_country,
+                random_seed=random_seed,
+                verbose=verbose,
+                n_jobs=n_jobs,
+            )
+            b_forecast = pd.concat([b_forecast, df_forecast.forecast])
+            b_forecast_up = pd.concat([b_forecast_up, df_forecast.upper_forecast])
+            b_forecast_low = pd.concat([b_forecast_low, df_forecast.lower_forecast])
+            # handle index being wrong for the flipped forecast which comes first
+            if n == 0:
+                b_forecast = b_forecast.iloc[::-1]
+                b_forecast_up = b_forecast_up.iloc[::-1]
+                b_forecast_low = b_forecast_low.iloc[::-1]
+                b_forecast.index = result_idx
+                b_forecast_up.index = result_idx
+                b_forecast_low.index = result_idx
+        except Exception as e:
+            print(f"back_forecast split {n} failed with {repr(e)}")
+            b_df = pd.DataFrame(
+                np.nan, index=df.index[int_idx:int_idx_1], columns=df.columns
+            )
+            b_forecast = pd.concat([b_forecast, b_df])
+            b_forecast_up = pd.concat([b_forecast_up, b_df])
+            b_forecast_low = pd.concat([b_forecast_low, b_df])
+
+    df_forecast.forecast = b_forecast
+    df_forecast.upper_forecast = b_forecast_up
+    df_forecast.lower_forecast = b_forecast_low
+    return df_forecast
+
+
+def remove_leading_zeros(df):
+    """Accepts wide dataframe, returns dataframe with zeroes preceeding any non-zero value as NaN."""
+    # keep the last row unaltered to keep metrics happier if all zeroes
+    temp = df.head(df.shape[0] - 1)
+    temp = temp.abs().cumsum(axis=0).replace(0, np.nan)
+    temp = df[~temp.isna()]
+    temp = temp.head(df.shape[0] - 1)
+    return pd.concat([temp, df.tail(1)], axis=0)
