@@ -24,6 +24,8 @@ from autots.evaluator.auto_model import (
     generate_score_per_series,
     model_forecast,
     validation_aggregation,
+    back_forecast,
+    remove_leading_zeros,
 )
 from autots.models.ensemble import (
     EnsembleTemplateGenerator,
@@ -88,6 +90,13 @@ class AutoTS(object):
     Attributes:
         best_model (pandas.DataFrame): DataFrame containing template for the best ranked model
         regression_check (bool): If True, the best_model uses an input 'User' future_regressor
+
+    Methods:
+        fit, predict
+        export_template, import_template, import_results
+        results, failure_rate
+        horizontal_to_df, mosaic_to_df
+        plot_horizontal, plot_horizontal_transformers, plot_generation_loss
     """
 
     def __init__(
@@ -305,6 +314,9 @@ class AutoTS(object):
             else ['ID'] + self.template_cols
         )
         self.initial_results = TemplateEvalObject()
+        self.best_model_name = ""
+        self.best_model_params = ""
+        self.best_model_transformation_params = ""
 
         if verbose > 2:
             print('"Hello. Would you like to destroy some evil today?" - Sanderson')
@@ -315,7 +327,7 @@ class AutoTS(object):
             return "Uninitiated AutoTS object"
         else:
             try:
-                return f"Initiated AutoTS object with best model: \n{self.best_model['Model'].iloc[0]}\n{self.best_model['TransformationParameters'].iloc[0]}\n{self.best_model['ModelParameters'].iloc[0]}"
+                return f"Initiated AutoTS object with best model: \n{self.best_model_name}\n{self.best_model_transformation_params}\n{self.best_model_params}"
             except Exception:
                 return "Initiated AutoTS object"
 
@@ -474,12 +486,7 @@ class AutoTS(object):
 
         # replace any zeroes that occur prior to all non-zero values
         if self.remove_leading_zeroes:
-            # keep the last row unaltered to keep metrics happier if all zeroes
-            temp = df_wide_numeric.head(df_wide_numeric.shape[0] - 1)
-            temp = temp.abs().cumsum(axis=0).replace(0, np.nan)
-            temp = df_wide_numeric[~temp.isna()]
-            temp = temp.head(df_wide_numeric.shape[0] - 1)
-            df_wide_numeric = pd.concat([temp, df_wide_numeric.tail(1)], axis=0)
+            df_wide_numeric = remove_leading_zeros(df_wide_numeric)
 
         # remove other ensembling types if univariate
         if df_wide_numeric.shape[1] == 1:
@@ -499,7 +506,7 @@ class AutoTS(object):
         ensemble = self.ensemble
 
         # check if NaN in last row
-        nan_tail = df_wide_numeric.tail(1).isna().sum(axis=1).iloc[0] > 0
+        self._nan_tail = df_wide_numeric.tail(1).isna().sum(axis=1).iloc[0] > 0
 
         self.df_wide_numeric = df_wide_numeric
         self.startTimeStamps = df_wide_numeric.notna().idxmax()
@@ -705,6 +712,7 @@ class AutoTS(object):
                     model_interrupt=self.model_interrupt,
                     grouping_ids=self.grouping_ids,
                     random_seed=random_seed,
+                    current_generation=(current_generation + 1),
                     verbose=verbose,
                     n_jobs=self.n_jobs,
                 )
@@ -890,9 +898,8 @@ class AutoTS(object):
                     val_future_regressor_test = []
 
                 # force NaN for robustness
-                if self.introduce_na or (self.introduce_na is None and nan_tail):
+                if self.introduce_na or (self.introduce_na is None and self._nan_tail):
                     nan_frac = val_df_train.shape[1] / num_validations
-                    int(nan_frac * y), int(nan_frac * (y + 1))
                     val_df_train.iloc[
                         -1, int(nan_frac * y) : int(nan_frac * (y + 1))
                     ] = np.nan
@@ -903,7 +910,6 @@ class AutoTS(object):
                     df_train=val_df_train,
                     df_test=val_df_test,
                     weights=current_weights,
-                    # model_count=model_count,
                     forecast_length=forecast_length,
                     frequency=frequency,
                     prediction_interval=prediction_interval,
@@ -1090,6 +1096,12 @@ or otherwise increase models available."""
                 self.ensemble_check = int((self.best_model['Ensemble'].iloc[0]) > 0)
             except IndexError:
                 raise ValueError(error_msg_template)
+        # give a more convenient dict option
+        self.best_model_name = self.best_model['Model'].iloc[0]
+        self.best_model_params = json.loads(self.best_model['ModelParameters'].iloc[0])
+        self.best_model_transformation_params = json.loads(
+            self.best_model['TransformationParameters'].iloc[0]
+        )
 
         # set flags to check if regressors or ensemble used in final model.
         param_dict = json.loads(self.best_model.iloc[0]['ModelParameters'])
@@ -1192,6 +1204,7 @@ or otherwise increase models available."""
                     grouping_ids=self.grouping_ids,
                     random_seed=self.random_seed,
                     verbose=verbose,
+                    n_jobs=self.n_jobs,
                     template_cols=self.template_cols,
                 )
                 # convert categorical back to numeric
@@ -1223,6 +1236,7 @@ or otherwise increase models available."""
                 grouping_ids=self.grouping_ids,
                 random_seed=self.random_seed,
                 verbose=verbose,
+                n_jobs=self.n_jobs,
                 template_cols=self.template_cols,
             )
             # convert categorical back to numeric
@@ -1322,6 +1336,9 @@ or otherwise increase models available."""
                 export_template = export_template.nsmallest(n, columns=['Score'])
                 if not include_results:
                     export_template = export_template[self.template_cols]
+                    export_template = pd.concat(
+                        [self.best_model, export_template]
+                    ).drop_duplicates()
         else:
             raise ValueError("`models` must be 'all' or 'best'")
         try:
@@ -1440,13 +1457,57 @@ or otherwise increase models available."""
             self.initial_results = self.initial_results.concat(new_obj)
         return self
 
+    def back_forecast(
+        self, column=None, n_splits: int = 3, tail: int = None, verbose: int = 0
+    ):
+        """Create forecasts for the historical training data, ie. backcast or back forecast.
+
+        This actually forecasts on historical data, these are not fit model values as are often returned by other packages.
+        As such, this will be slower, but more representative of real world model performance.
+        There may be jumps in data between chunks.
+
+        Args are same as for model_forecast except...
+        n_splits(int): how many pieces to split data into. Pass 2 for fastest, or "auto" for best accuracy
+        column (str): if to run on only one column, pass column name. Faster than full.
+        tail (int): df.tail() of the dataset, back_forecast is only run on n most recent observations.
+
+        Returns a standard prediction object (access .forecast, .lower_forecast, .upper_forecast)
+        """
+        if self.best_model.empty:
+            raise ValueError("No best_model. AutoTS .fit() needs to be run.")
+        if column is not None:
+            input_df = pd.DataFrame(self.df_wide_numeric[column])
+        else:
+            input_df = self.df_wide_numeric
+        if tail is not None:
+            input_df = input_df.tail(tail)
+        result = back_forecast(
+            df=input_df,
+            model_name=self.best_model_name,
+            model_param_dict=self.best_model_params,
+            model_transform_dict=self.best_model_transformation_params,
+            future_regressor_train=self.future_regressor_train,
+            n_splits=n_splits,
+            forecast_length=self.forecast_length,
+            frequency=self.frequency,
+            prediction_interval=self.prediction_interval,
+            no_negatives=self.no_negatives,
+            constraint=self.constraint,
+            holiday_country=self.holiday_country,
+            random_seed=self.random_seed,
+            n_jobs=self.n_jobs,
+            verbose=verbose,
+        )
+        return result
+
     def horizontal_to_df(self):
         """helper function for plotting."""
         if self.best_model.empty:
             raise ValueError("No best_model. AutoTS .fit() needs to be run.")
         if self.best_model['Ensemble'].iloc[0] != 2:
             raise ValueError("Only works on horizontal ensemble type models.")
-        series = json.loads(self.best_model['ModelParameters'].iloc[0])['series']
+        ModelParameters = self.best_model_params
+        series = ModelParameters['series']
         series = pd.DataFrame.from_dict(series, orient="index").reset_index(drop=False)
         if series.shape[1] > 2:
             # for mosaic style ensembles, choose the mode model id
@@ -1463,6 +1524,22 @@ or otherwise increase models available."""
             self.df_wide_numeric.mean().to_frame(), right_index=True, left_on="Series"
         )
         series.columns = ["Series", "ID", 'Model', "Volatility", "Mean"]
+        series['Transformers'] = series['ID'].copy()
+        series['FillNA'] = series['ID'].copy()
+        lookup = {}
+        na_lookup = {}
+        for k, v in ModelParameters['models'].items():
+            try:
+                trans_params = json.loads(v.get('TransformationParameters', '{}'))
+                lookup[k] = ",".join(trans_params.get('transformations', {}).values())
+                na_lookup[k] = trans_params.get('fillna', '')
+            except Exception:
+                lookup[k] = "None"
+                na_lookup[k] = "None"
+        series['Transformers'] = (
+            series['Transformers'].replace(lookup).replace("", "None")
+        )
+        series['FillNA'] = series['FillNA'].replace(na_lookup).replace("", "None")
         return series
 
     def mosaic_to_df(self):
@@ -1471,7 +1548,7 @@ or otherwise increase models available."""
             raise ValueError("No best_model. AutoTS .fit() needs to be run.")
         if self.best_model['Ensemble'].iloc[0] != 2:
             raise ValueError("Only works on horizontal ensemble type models.")
-        ModelParameters = json.loads(self.best_model['ModelParameters'].iloc[0])
+        ModelParameters = self.best_model_params
         if str(ModelParameters['model_name']).lower() != 'mosaic':
             raise ValueError("Only works on mosaic ensembles.")
         series = pd.DataFrame.from_dict(ModelParameters['series'])
@@ -1500,6 +1577,118 @@ or otherwise increase models available."""
             'log(Volatility)'
         ].plot(style='o', **kwargs)
 
+    def plot_horizontal_transformers(
+        self, method="transformers", color_list=None, **kwargs
+    ):
+        """Simple plot to visualize transformers used.
+        Note this doesn't capture transformers nested in simple ensembles.
+
+        Args:
+            method (str): 'fillna' or 'transformers' - which to plot
+            color_list = list of colors to *sample* for bar colors. Can be names or hex.
+            **kwargs passed to pandas.plot()
+        """
+        series = self.horizontal_to_df()
+        if str(method).lower() == "fillna":
+            transformers = series['FillNA'].value_counts()
+        else:
+            transformers = pd.Series(
+                ",".join(series['Transformers']).split(",")
+            ).value_counts()
+        if color_list is None:
+            color_list = colors_list
+        colors = random.sample(color_list, transformers.shape[0])
+        # plot
+        transformers.plot(kind='bar', color=colors, **kwargs)
+
+    def plot_generation_loss(self, **kwargs):
+        """Plot improvement in accuracy over generations.
+        Note: this is only "one size fits all" accuracy and
+        doesn't account for the benefits seen for ensembling.
+
+        Args:
+            **kwargs passed to pd.DataFrame.plot()
+        """
+        for_gens = self.initial_results.model_results[
+            (self.initial_results.model_results['ValidationRound'] == 0)
+            & (self.initial_results.model_results['Ensemble'] < 1)
+        ]
+        for_gens.groupby("Generation")['Score'].min().cummin().plot(
+            ylabel="Lowest Score", **kwargs
+        )
+
+    def plot_backforecast(
+        self, series=None, n_splits: int = 3, start_date=None, **kwargs
+    ):
+        """Plot the historical data and fit forecast on historic.
+
+        Args:
+            series (str or list): column names of time series
+            n_splits (int or str): "auto", number > 2, higher more accurate but slower
+            **kwargs passed to pd.DataFrame.plot()
+        """
+        if series is None:
+            series = random.choice(self.df_wide_numeric.columns)
+        b_df = self.back_forecast(column=series, n_splits=n_splits, verbose=0).forecast
+        b_df = b_df.rename(columns=lambda x: str(x) + "_forecast")
+        plot_df = pd.concat(
+            [
+                pd.DataFrame(self.df_wide_numeric[series]),
+                b_df,
+            ],
+            axis=1,
+        )
+        if start_date is not None:
+            plot_df = plot_df[plot_df.index >= start_date]
+        plot_df = remove_leading_zeros(plot_df)
+        plot_df.plot(**kwargs)
+
+
+colors_list = [
+    '#FF00FF',
+    '#7FFFD4',
+    '#00FFFF',
+    '#F5DEB3',
+    '#FF6347',
+    '#8B008B',
+    '#696969',
+    '#FFC0CB',
+    '#C71585',
+    '#008080',
+    '#663399',
+    '#32CD32',
+    '#66CDAA',
+    '#A9A9A9',
+    '#2F4F4F',
+    '#FFDEAD',
+    '#800000',
+    '#FDF5E6',
+    '#F5F5F5',
+    '#F0FFF0',
+    '#87CEEB',
+    '#A52A2A',
+    '#90EE90',
+    '#7FFF00',
+    '#E9967A',
+    '#1E90FF',
+    '#FFF0F5',
+    '#ADD8E6',
+    '#008B8B',
+    '#FFF5EE',
+    '#00FA9A',
+    '#9370DB',
+    '#4682B4',
+    '#006400',
+    '#AFEEEE',
+    '#CD853F',
+    '#9400D3',
+    '#EE82EE',
+    '#00008B',
+    '#4B0082',
+    '#0403A7',
+    "#000000",
+]
+
 
 class AutoTSIntervals(object):
     """Autots looped to test multiple prediction intervals. Experimental.
