@@ -230,8 +230,8 @@ class Detrend(EmptyTransformer):
         except Exception:
             raise ValueError("Data Cannot Be Converted to Numeric Float")
 
-        Y = df.values
-        X = pd.to_numeric(df.index, errors='coerce', downcast='integer').values
+        Y = df.to_numpy()
+        X = pd.to_numeric(df.index, errors='coerce', downcast='integer').to_numpy()
         if self.model == 'GLS':
             from statsmodels.regression.linear_model import GLS
 
@@ -242,7 +242,7 @@ class Detrend(EmptyTransformer):
                 self.trnd_trans = PositiveShift(
                     log=False, center_one=True, squared=False
                 )
-                Y = pd.DataFrame(self.trnd_trans.fit_transform(df)).values
+                Y = pd.DataFrame(self.trnd_trans.fit_transform(df)).to_numpy()
             X = X.reshape((-1, 1))
             self.trained_model.fit(X, Y)
         self.shape = df.shape
@@ -319,28 +319,19 @@ class StatsmodelsFilter(EmptyTransformer):
     """Irreversible filters.
 
     Args:
-        method (str): bkfilter or cffilter
+        method (str): bkfilter or cffilter or convolution_filter
     """
 
     def __init__(self, method: str = 'bkfilter', **kwargs):
         super().__init__(name="StatsmodelsFilter")
         self.method = method
 
-    def fit(self, df):
-        """Fits filter.
-
-        Args:
-            df (pandas.DataFrame): input dataframe
-        """
-        return self
-
     def fit_transform(self, df):
         """Fit and Return Detrended DataFrame.
 
         Args:
             df (pandas.DataFrame): input dataframe
         """
-        self.fit(df)
         return self.transform(df)
 
     def transform(self, df):
@@ -349,11 +340,6 @@ class StatsmodelsFilter(EmptyTransformer):
         Args:
             df (pandas.DataFrame): input dataframe
         """
-        try:
-            df = df.astype(float)
-        except Exception:
-            raise ValueError("Data Cannot Be Converted to Numeric Float")
-
         if self.method == 'bkfilter':
             from statsmodels.tsa.filters import bk_filter
 
@@ -366,16 +352,139 @@ class StatsmodelsFilter(EmptyTransformer):
             cycle, trend = cf_filter.cffilter(df)
             cycle.columns = df.columns
             df = df - cycle
+        elif "convolution_filter" in self.method:
+            from statsmodels.tsa.filters.filtertools import convolution_filter
+
+            df = convolution_filter(df, [[0.75] * df.shape[1], [0.25] * df.shape[1]])
+            df = df.fillna(method='ffill').fillna(method='bfill')
         return df
 
-    def inverse_transform(self, df):
-        """Return data to original form.
+
+class HPFilter(EmptyTransformer):
+    """Irreversible filters.
+
+    Args:
+        lamb (int): lambda for hpfilter
+    """
+
+    def __init__(self, part: str = 'trend', lamb: float = 1600, **kwargs):
+        super().__init__(name="HPFilter")
+        self.part = part
+        self.lamb = lamb
+
+    def fit_transform(self, df):
+        """Fit and Return Detrended DataFrame.
 
         Args:
             df (pandas.DataFrame): input dataframe
         """
+        return self.transform(df)
+
+    def transform(self, df):
+        """Return detrended data.
+
+        Args:
+            df (pandas.DataFrame): input dataframe
+        """
+        from statsmodels.tsa.filters.hp_filter import hpfilter
+
+        def _hpfilter_one_return(series, lamb=1600, part="trend"):
+            """Convert filter to apply on pd DataFrame."""
+            hp_cycle, hp_trend = hpfilter(series, lamb)
+            if part == "cycle":
+                return hp_cycle
+            else:
+                return hp_trend
+
+        if df.isnull().values.any():
+            raise ValueError("hpfilter does not handle null values.")
+        df = df.apply(_hpfilter_one_return, lamb=self.lamb, part=self.part)
         return df
 
+    @staticmethod
+    def get_new_params(method: str = 'random'):
+        part = random.choices(['trend', 'cycle'], weights=[0.98, 0.02])[0]
+        lamb = random.choices(
+            [1600, 6.25, 129600, 104976000000], weights=[0.5, 0.2, 0.2, 0.1]
+        )[0]
+        return {"part": part, "lamb": lamb}
+
+
+class STLFilter(EmptyTransformer):
+    """Irreversible filters.
+
+    Args:
+        decomp_type (str): which decomposition to use
+        part (str): which part of decomposition to return
+        seaonal (int): seaonsal component of STL
+    """
+
+    def __init__(
+        self, decomp_type="STL", part: str = 'trend', seasonal: int = 7, **kwargs
+    ):
+        super().__init__(name="STLFilter")
+        self.part = part
+        self.seasonal = seasonal
+        self.decomp_type = decomp_type
+
+    def fit_transform(self, df):
+        """Fit and Return Detrended DataFrame.
+
+        Args:
+            df (pandas.DataFrame): input dataframe
+        """
+        return self.transform(df)
+
+    def transform(self, df):
+        """Return detrended data.
+
+        Args:
+            df (pandas.DataFrame): input dataframe
+        """
+        from statsmodels.tsa.seasonal import STL, seasonal_decompose
+
+        def _stl_one_return(series, decomp_type="STL", seasonal=7, part="trend"):
+            """Convert filter to apply on pd DataFrame."""
+            if str(decomp_type).lower() == 'stl':
+                result = STL(series, seasonal=seasonal).fit()
+            else:
+                result = seasonal_decompose(series)
+            if part == "seasonal":
+                return result.seasonal
+            elif part == "resid":
+                return result.resid
+            else:
+                return result.trend
+
+        if df.isnull().values.any():
+            raise ValueError("STLFilter does not handle null values.")
+
+        df = df.apply(
+            _stl_one_return,
+            decomp_type=self.decomp_type,
+            seasonal=self.seasonal,
+            part=self.part,
+        )
+        return df.fillna(method='ffill').fillna(method='bfill')
+
+    @staticmethod
+    def get_new_params(method: str = 'random'):
+        decomp_type = random.choices(['STL', 'seasonal_decompose'], weights=[0.5, 0.5])[
+            0
+        ]
+        part = random.choices(
+            ['trend', 'seasonal', "resid"], weights=[0.98, 0.02, 0.001]
+        )[0]
+        if decomp_type == "STL":
+            seasonal = seasonal_int()
+            if seasonal < 7:
+                seasonal = 7
+            elif seasonal % 2 == 0:
+                seasonal = seasonal - 1
+            return {"decomp_type": decomp_type, "part": part, "seasonal": seasonal}
+        else:
+            return {"decomp_type": decomp_type, "part": part}
+
 
 class SinTrend(EmptyTransformer):
     """Modelling sin."""
@@ -766,30 +875,6 @@ class RollingMeanTransformer(EmptyTransformer):
                 return staged
 
 
-"""
-df = df_wide_numeric.tail(60).head(50).fillna(0)
-df_forecast = (df_wide_numeric).tail(10).fillna(0)
-forecats = transformed.tail(10)
-test = RollingMeanTransformer().fit(df)
-transformed = test.transform(df)
-inverse = test.inverse_transform(forecats, trans_method = 'forecast')
-df == test.inverse_transform(test.transform(df), trans_method = 'original')
-inverse == df_wide_numeric.tail(10)
-"""
-"""
-df = df_wide_numeric.tail(60).fillna(0)
-test = SeasonalDifference().fit(df)
-transformed = test.transform(df)
-forecats = transformed.tail(10)
-df == test.inverse_transform(transformed, trans_method = 'original')
-
-df = df_wide_numeric.tail(60).head(50).fillna(0)
-test = SeasonalDifference().fit(df)
-inverse = test.inverse_transform(forecats, trans_method = 'forecast')
-inverse == df_wide_numeric.tail(10).fillna(0)
-"""
-
-
 class SeasonalDifference(EmptyTransformer):
     """Remove seasonal component.
 
@@ -931,6 +1016,8 @@ class DatepartRegressionTransformer(EmptyTransformer):
             raise ValueError("Data Cannot Be Converted to Numeric Float")
 
         y = df.values
+        if y.shape[1] == 1:
+            y = y.ravel()
         X = date_part(df.index, method=self.datepart_method)
         from autots.models.sklearn import retrieve_regressor
 
@@ -1021,8 +1108,6 @@ class DifferencedTransformer(EmptyTransformer):
         Args:
             df (pandas.DataFrame): input dataframe
         """
-        # df = df_wide_numeric.tail(60).head(50)
-        # df_forecast = (df_wide_numeric - df_wide_numeric.shift(1)).tail(10)
         df = (df - df.shift(self.lag)).fillna(method='bfill')
         return df
 
@@ -1051,6 +1136,8 @@ class DifferencedTransformer(EmptyTransformer):
         else:
             df_len = df.shape[0]
             df = pd.concat([self.last_values, df], axis=0)
+            if df.isnull().values.any():
+                raise ValueError("NaN in DifferencedTransformer.inverse_transform")
             return df.cumsum().tail(df_len)
 
 
@@ -1646,6 +1733,183 @@ class CenterLastValue(EmptyTransformer):
         return self.transform(df)
 
 
+class ScipyFilter(EmptyTransformer):
+    """Irreversible filters from Scipy
+
+    Args:
+        method (str): "hilbert", "wiener", "savgol_filter", "butter", "cheby1", "cheby2", "ellip", "bessel",
+        method_args (list): passed to filter as appropriate
+    """
+
+    def __init__(self, method: str = 'hilbert', method_args: list = None, **kwargs):
+        super().__init__(name="ScipyFilter")
+        self.method = method
+        self.method_args = method_args
+
+    def fit(self, df):
+        """Fits filter.
+
+        Args:
+            df (pandas.DataFrame): input dataframe
+        """
+        return self
+
+    def fit_transform(self, df):
+        """Fit and Return Detrended DataFrame.
+
+        Args:
+            df (pandas.DataFrame): input dataframe
+        """
+        self.fit(df)
+        return self.transform(df)
+
+    @staticmethod
+    def get_new_params(method: str = 'random'):
+        method = random.choices(
+            [
+                "hilbert",
+                "wiener",
+                "savgol_filter",
+                "butter",
+                # "cheby1",
+                # "cheby2",
+                # "ellip",
+                # "bessel",
+            ],
+            [0.1, 0.2, 0.2, 0.1],
+            k=1,
+        )[0]
+        # analog_choice = bool(random.randint(0, 1))
+        analog_choice = False
+        xn = random.randint(1, 99)
+        btype = random.choice(["lowpass", "highpass"])  # "bandpass", "bandstop"
+        if method in ['wiener', 'hilbert']:
+            method_args = None
+        elif method == "savgol_filter":
+            method_args = [random.randrange(5, 11, 2), random.randint(1, 3)]
+        elif method in ["butter", "bessel"]:
+            if btype in ["bandpass", "bandstop"]:
+                Wn = [xn / 100, random.randint(1, 99) / 100]
+            else:
+                Wn = xn / 100 if not analog_choice else xn
+            method_args = [
+                random.randint(1, 5),
+                Wn,
+                btype,
+                analog_choice,
+            ]
+        elif method in ["cheby1", "cheby2"]:
+            if btype in ["bandpass", "bandstop"]:
+                Wn = [xn / 100, random.randint(1, 99) / 100]
+            else:
+                Wn = xn / 100 if not analog_choice else xn
+            method_args = [
+                random.randint(1, 5),
+                random.randint(1, 10),
+                Wn,
+                btype,
+                analog_choice,
+            ]
+        elif method in ["ellip"]:
+            if btype in ["bandpass", "bandstop"]:
+                Wn = [xn / 100, random.randint(1, 99) / 100]
+            else:
+                Wn = xn / 100 if not analog_choice else xn
+            method_args = [
+                random.randint(1, 5),
+                random.randint(1, 10),
+                random.randint(1, 10),
+                Wn,
+                btype,
+                analog_choice,
+            ]
+        return {
+            "method": method,
+            "method_args": method_args,
+        }
+
+    def transform(self, df):
+        """Return detrended data.
+
+        Args:
+            df (pandas.DataFrame): input dataframe
+        """
+
+        if self.method == 'hilbert':
+            from scipy.signal import hilbert
+
+            test = pd.DataFrame(hilbert(df.values), columns=df.columns, index=df.index)
+            return np.abs(test)
+        elif self.method == 'wiener':
+            from scipy.signal import wiener
+
+            return pd.DataFrame(wiener(df.values), columns=df.columns, index=df.index)
+        elif self.method == 'savgol_filter':
+            from scipy.signal import savgol_filter
+
+            # args = [5, 2]
+            return pd.DataFrame(
+                savgol_filter(df.values, *self.method_args, axis=0, mode='nearest'),
+                columns=df.columns,
+                index=df.index,
+            )
+        elif self.method == 'butter':
+            from scipy.signal import butter, sosfiltfilt
+
+            # args = [4, 0.125]
+            # [4, 100, 'lowpass'], [1, 0.125, "highpass"]
+            sos = butter(*self.method_args, output='sos')
+            return pd.DataFrame(
+                sosfiltfilt(sos, df.values, axis=0), columns=df.columns, index=df.index
+            )
+        elif self.method == "cheby1":
+            from scipy.signal import cheby1, sosfiltfilt
+
+            # args = [4, 5, 100, 'lowpass', True]
+            # args = [10, 1, 15, 'highpass']
+            sos = cheby1(*self.method_args, output='sos')
+            return pd.DataFrame(
+                sosfiltfilt(sos, df.values, axis=0), columns=df.columns, index=df.index
+            )
+        elif self.method == "cheby2":
+            from scipy.signal import cheby2, sosfiltfilt
+
+            # args = [4, 40, 100, 'lowpass', True]
+            # args = [12, 20, 17, 'highpass']
+            sos = cheby2(*self.method_args, output='sos')
+            return pd.DataFrame(
+                sosfiltfilt(sos, df.values, axis=0), columns=df.columns, index=df.index
+            )
+        elif self.method == "ellip":
+            from scipy.signal import ellip, sosfiltfilt
+
+            # args = [4, 5, 40, 100, 'lowpass', True]
+            # args = [8, 1, 100, 17, 'highpass']
+            sos = ellip(*self.method_args, output='sos')
+            return pd.DataFrame(
+                sosfiltfilt(sos, df.values, axis=0), columns=df.columns, index=df.index
+            )
+        elif self.method == "bessel":
+            from scipy.signal import bessel, sosfiltfilt
+
+            # args = [4, 100, 'lowpass', True]
+            # args = [3, 10, 'highpass']
+            sos = bessel(*self.method_args, output='sos')
+            return pd.DataFrame(
+                sosfiltfilt(sos, df.values, axis=0), columns=df.columns, index=df.index
+            )
+        else:
+            raise ValueError(f"ScipyFilter method {self.method} not found.")
+
+    def inverse_transform(self, df):
+        """Return data to original form.
+
+        Args:
+            df (pandas.DataFrame): input dataframe
+        """
+        return df
+
+
 # lookup dict for all non-parameterized transformers
 trans_dict = {
     'None': EmptyTransformer(),
@@ -1663,6 +1927,7 @@ trans_dict = {
     'SeasonalDifference28': SeasonalDifference(lag_1=28, method='Mean'),
     'bkfilter': StatsmodelsFilter(method='bkfilter'),
     'cffilter': StatsmodelsFilter(method='cffilter'),
+    'convolution_filter': StatsmodelsFilter(method='convolution_filter'),
     "Discretize": Discretize(discretization="center", n_bins=10),
     'DatepartRegressionLtd': DatepartRegressionTransformer(
         regression_model={
@@ -1690,6 +1955,9 @@ have_params = {
     'Round': Round,
     'Slice': Slice,
     'Detrend': Detrend,
+    'ScipyFilter': ScipyFilter,
+    'HPFilter': HPFilter,
+    'STLFilter': STLFilter,
 }
 # where will results will vary if not all series are included together
 shared_trans = ['PCA', 'FastICA']
@@ -1762,6 +2030,9 @@ class GeneralTransformer(object):
             'ClipOutliers' - remove outliers
             'Discretize' - bin or round data into groups
             'DatepartRegression' - move a trend trained on datetime index
+            "ScipyFilter" - filter data (lose information but smoother!) from scipy
+            "HPFilter" - statsmodels hp_filter
+            "STLFilter" - seasonal decompose and keep just one part of decomposition.
 
         transformation_params (dict): params of transformers {0: {}, 1: {'model': 'Poisson'}, ...}
             pass through dictionary of empty dictionaries to utilize defaults
@@ -2088,8 +2359,8 @@ def get_transformer_params(transformer: str = "EmptyTransformer", method: str =
 transformer_dict = {
     None: 0.0,
     'MinMaxScaler': 0.05,
-    'PowerTransformer': 0.1,
-    'QuantileTransformer': 0.1,
+    'PowerTransformer': 0.05,
+    'QuantileTransformer': 0.05,
     'MaxAbsScaler': 0.05,
     'StandardScaler': 0.04,
     'RobustScaler': 0.05,
@@ -2106,10 +2377,12 @@ transformer_dict = {
     'Log': 0.01,
     'IntermittentOccurrence': 0.01,
     # 'SeasonalDifference7': 0.0,  # old
-    'SeasonalDifference': 0.08,
+    'SeasonalDifference': 0.1,
     # 'SeasonalDifference28': 0.0,  # old
     'cffilter': 0.01,
     'bkfilter': 0.05,
+    'convolution_filter': 0.001,
+    "HPFilter": 0.02,
     'DatepartRegression': 0.02,
     # 'DatepartRegressionElasticNet': 0.0,  # old
     # 'DatepartRegressionLtd': 0.0,  # old
@@ -2117,13 +2390,17 @@ transformer_dict = {
     "Discretize": 0.05,
     "CenterLastValue": 0.01,
     "Round": 0.05,
-    "Slice": 0.01,
+    "Slice": 0.02,
+    "ScipyFilter": 0.02,
+    "STLFilter": 0.01,
 }
 # remove any slow transformers
 fast_transformer_dict = transformer_dict.copy()
 del fast_transformer_dict['DatepartRegression']
 del fast_transformer_dict['SinTrend']
 del fast_transformer_dict['FastICA']
+del fast_transformer_dict['ScipyFilter']
+del fast_transformer_dict['STLFilter']
 
 # and even more
 superfast_transformer_dict = fast_transformer_dict.copy()
@@ -2131,6 +2408,8 @@ del superfast_transformer_dict['IntermittentOccurrence']
 del superfast_transformer_dict['cffilter']
 del superfast_transformer_dict['QuantileTransformer']
 del superfast_transformer_dict['PowerTransformer']
+del fast_transformer_dict['convolution_filter']
+del fast_transformer_dict['HPFilter']
 
 # probability dictionary of FillNA methods
 na_probs = {
